
;	802 implementation of Micheners	circle algorithm.


	asect	0
	org	0x1400


;	.GLOBL	CIRCLE		; Enter	here to	get radius from	host.
;	.GLOBL	FATCIR		; As above, for	fat circle.
;	.GLOBL	CIR		; Enter	here if	caller has already put
				; radius in RADIUS and initialized
				; the fat circle flag FCFLAG.

;	Local variables	:

RADIUS:	ds	2		; Radius of circle.
FCFLAG:	ds	2		; -1 for fat circle.

IX:	ds	2		; IX and IY are	the displacements
IY:	ds	2		; relative to XPOS, YPOS of the	points
				; on the circumference of the circle.
D:	ds	2		; Decision var,	used to	determine
				; whether next point is	on same	line
				; or the one below.

CXPX:	ds	2		; These	are the	translated points
CXMX:	ds	2		; (XPOS,YPOS) +- (IX,IY) and
CYPY:	ds	2		; (XPOS, YPOS) +- (IY, IX).
CYMY:	ds	2
CYPX:	ds	2
CYMX:	ds	2
CXPY:	ds	2
CXMY:	ds	2


IX4P6:	ds	2		; 4 * IX + 6
M4IYP4:	ds	2		; -4 * IY + 4

IX2P1	ds	2		; IX * 2 + 1
IY2P1	ds	2		; IY * 2 + 1


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;	This is	an implementation of Micheners circle drawing		;
;	algorithm, described in	Foley/VanDam, pg445.			;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

FATCIR:	lda	#-1		; Entry	for fat	circle from host.
	bra	CIRC1

CIRCLE:	lda	#0		; Entry	for regular circle from	host.
CIRC1:	sta	FCFLAG

	jsr	.GET2		; Get radius from host.
	sty	RADIUS+1	; High byte in Y reg.
	stx	RADIUS		; Low byte in X	reg.


CIR:				; Set X counter down, Y counter up.
	lda	#0377-040
	and	<0x24
	sta	<0x24

;	jsr	set802		; turn off emulation mode.
	sei
	clc
	xce
	rep	#0x30		; 16 bit regies.

	stz	IX		; init IX = 0, IY = RADIUS.
	lda	RADIUS
	sta	IY

	asl	a		; for computation below.
;	set D	= 3 - 2*RADIUS
;		= 3 - acc
;		= -acc + 3
;		= ((acc	eor -1)+1) + 3
;		= (acc eor -1) +4

	eor	##-1
	clc
	adc	##4
	sta	D


	lda	##6		; Set IX4P6 = IX*4 + 6
	sta	IX4P6

;	Set M4IYP4 = -4*IY + 4 = 4(1-IY)

	lda	IY		; set acc = 1 -	IY
	eor	##-1		; = 1 +	(-IY)
	inc	a		; = 1 +	(1+ (IY	eor -1))
	inc	a		; = 2 +	(IY eor	-1)

	asl	a		; set M4IYP4 = 4*acc
	asl	a		; = 4(1-IY)
	sta	M4IYP4

;	Init the 8 coordinates (XPOS,YPOS) +- (IX, IY)
;	and (XPOS, YPOS) +- (IY, IX).
;	Remember at this point IX = 0.

	lda	<XPOS
	sta	CXPX		; CXMX = CXPX =	XPOS
	sta	CXMX

	clc			; CXPY = XPOS +	IY
	adc	IY
	sta	CXPY

	lda	<YPOS
	sta	CYPX		; CYPX = CYMX =	YPOS
	sta	CYMX


	clc			; CYPY = YPOS +	IY
	adc	IY
	sta	CYPY

	sec
	lda	<XPOS		; CXMY = XPOS -	IY
	sbc	IY
	sta	CXMY

	sec
	lda	<YPOS		; CYMY = YPOS -	IY
	sbc	IY
	sta	CYMY

	lda	##1
	sta	IX2P1
	lda	IY
	asl	a
	inc	a
	sta	IY2P1
	
;	Done initializing, now we can start
;	drawing	the circle.

LOOP
	bit	D
	bpl	XY		; Branch if D positive to drop
				; to next line.
	ldx	CXPY
	ldy	CYPX
	lda	IY2P1
	bsl	PLOT

	ldx	CXPY
	ldy	CYMX
	lda	IY2P1
	bsl	PLOT

	
	clc
	lda	D		; D += IX4P6.  Update decision variable.
	adc	IX4P6
	sta	D

	inc	IX		; IX++.
	inc	IX2P1
	inc	IX2P1
	
	inc	CXPX		; Update the coordinates that
	inc	CYPX		; are translates of IX.
	dec	CXMX
	dec	CYMX

	clc
	lda	IX4P6		; IX4P6	+= 4.
	adc	##4
	sta	IX4P6

	lda	IX		; Compare IX, IY.
	cmp	IY
	bcc	LOOP		; LOOP if IX < IY.
	bne	$1		; Return if IX > IY.
	jsr	PLOT		; Plot 1 last point if IX = IY,	and return.
$1	jmp	set8bit
	
;	Here when it's time to move CAP	in Y as	well as	X.

XY
	ldx	CXPX
	ldy	CYPY
	lda	IX2P1
	bsl	PLOT

	ldx	CXPX
	ldy	CYMY
	lda	IX2P1
	bsl	PLOT

	ldx	CXPY
	ldy	CYPX
	lda	IY2P1
	bsl	PLOT

	ldx	CXPY
	ldy	CYMX
	lda	IY2P1
	bsl	PLOT

		
;	 Set D = 4*(IX-IY) + 10.
;	       = IX4P6 + M4IYP4.

	lda	D
	clc
	adc	IX4P6		; D += IX4P6.  Update decision variable.
	clc
	adc	M4IYP4		; D += M4IYP4.
	sta	D

	inc	IX		; IX++.
	inc	IX2P1
	inc	IX2P1

	dec	IY		; IY--.
	dec	IY2P1
	dec	IY2P1

	inc	CXPX		; Update the coordinates that
	inc	CYPX		; are translates of IX.
	dec	CXMX
	dec	CYMX

	inc	CXMY		; Update the coordinates that
	inc	CYMY		; are translates of IY.
	dec	CXPY
	dec	CYPY

	clc
	lda	IX4P6		; IX4P6	+= 4.
	adc	##4
	sta	IX4P6

	clc
	lda	M4IYP4		; M4IYP4 += 4.
	adc	##4
	sta	M4IYP4

	lda	IX		; Compare IX, IY.
	cmp	IY
	bcs	$1		; LOOP if IX < IY.
	brl	LOOP
$1	bne	$2		; Return if IX > IY.

	ldx	CXPX
	ldy	CYPY
	lda	IX2P1
	bsl	PLOT		; Plot 1 last point if IX = IY,	and return.

	ldx	CXPX
	ldy	CYMY
	lda	IX2P1
	bsl	PLOT		; Plot 1 last point if IX = IY,	and return.

$2	jmp	set8bit		; get back in 6502 mode, return
				; to interpreter.


;	Draw in X direction from (X reg,Y reg) acc pixels.

PLOT

	stx	<XLO
	sty	<YLO
	tax

	sep	#0x20		; set 8	bit acc	for pixel writes.
	lda	<COLOR
$1
	sta	<VMIX
	dex
	bne	$1

	rep	#0x20
	rts
